@@ -3,8 +3,10 @@
 #include "string_manip.h"
 #include "VariableList.h"
 
-#include "gmp.h"
-#include "mpfr.h"
+#include "number.h"
+#ifdef MEMWATCH
+#include "memwatch.h"
+#endif
 
 @implementation VariableList
 // needs to be REALLY fast
@@ -51,14 +53,14 @@
 	if ([ch isEqualToString:@"value"]) {
 		char * input=strdup([anObject UTF8String]);
 		if (justnumbers(input)) {
-			mpfr_t la;
+			Number la;
 
-			mpfr_init_set(la, last_answer, GMP_RNDN);
+			num_init_set(la, last_answer);
 			parseme(input);
-			mpfr_set(theval->value, last_answer, GMP_RNDN);
+			num_set(theval->value, last_answer);
 			theval->exp = 0;
-			mpfr_set(last_answer, la, GMP_RNDN);
-			mpfr_clear(la);
+			num_set(last_answer, la);
+			num_free(la);
 		} else {
 			theval->expression = input;
 			theval->exp = 1;
@@ -74,9 +76,9 @@
 {
 	char varname[20];
 	int i=1;
-	mpfr_t blank;
+	Number blank;
 
-	mpfr_init_set_ui(blank,0,GMP_RNDN);
+	num_init_set_ui(blank,0);
 	sprintf(varname,"NewVariable%i",i);
 	while(varexists(varname)) {
 		++i;
@@ -104,4 +106,36 @@
     [theList reloadData];
 }
 
+- (IBAction)copyMe:(id)sender
+{
+    NSPasteboard* pboard=[NSPasteboard generalPasteboard];
+    NSString *theString = @"";
+    NSIndexSet* rowEnumerator=[theList selectedRowIndexes];
+    unsigned int theIndex=0;
+    
+    // Set the pasteboard types you want here.
+    [pboard declareTypes:[NSArray arrayWithObject:NSStringPboardType] owner:self];
+    
+    theIndex = [rowEnumerator firstIndex];
+    do {
+	struct variable *keyval = getrealnvar(theIndex);
+	if (keyval == NULL) {
+	    theString = [theString stringByAppendingString:@"BAD ROW\n"];
+	} else {
+	    theString = [theString stringByAppendingFormat:@"%s\t%s\n", keyval->key, keyval->exp?keyval->expression:print_this_result(keyval->value)];
+	}
+    } while ((theIndex = [rowEnumerator indexGreaterThanIndex:theIndex]) != NSNotFound);
+    [pboard setString:theString forType:NSStringPboardType];    
+}
+
+// ** Make sure we don't enable the copy menu item unless there is something to copy.
+- (BOOL)validateMenuItem:(id <NSMenuItem>)menuItem
+{
+    if ([menuItem tag]==666)
+    {
+	return ([theList numberOfSelectedRows]>0);
+    }
+    return YES;
+}
+
 @end
