@@ -292,6 +292,12 @@ static pthread_mutex_t displayLock;
 	[prefs setObject:@"4" forKey:@"initialized"];
 	[prefs setObject:@"1" forKey:@"engineeringNotation"];
     }
+    if (prefsVersion < 5) {
+	Dprintf("Upgrading prefs to v5\n");
+	[prefs setObject:@"5" forKey:@"initialized"];
+	[prefs setObject:@"" forKey:@"alternateInputDecimalSeparator"];
+	[prefs setObject:@"" forKey:@"alternateInputThousandsSeparator"];
+    }
     conf.precision = [prefs integerForKey:@"precision"];
     switch([prefs integerForKey:@"engineeringNotation"]) {
 		case 1: conf.engineering = automatic; break;
@@ -340,22 +346,35 @@ static pthread_mutex_t displayLock;
     just_answered = FALSE;
 
     /* Set up the character translation */
-    if ([[prefs objectForKey:NSDecimalSeparator] length] > 0) {
+    NSNumberFormatter *numFormat = [[[NSNumberFormatter alloc] init] autorelease];
+    if ([[numFormat decimalSeparator] length] > 0) {
 	Dprintf("NSDecimalSeparator > 0\n");
-	conf.dec_delimiter = [[prefs objectForKey:NSDecimalSeparator] characterAtIndex:0];
+	conf.dec_delimiter = [[numFormat decimalSeparator] characterAtIndex:0];
     } else {
 	conf.dec_delimiter = '.';
     }
-    Dprintf("NSDecimalSeparator set\n");
-    if ([[prefs objectForKey:NSThousandsSeparator] length] > 0) {
-	Dprintf("thou_delimiter length > 0\n"); fflush(NULL);
-	conf.thou_delimiter = [[prefs objectForKey:NSThousandsSeparator] characterAtIndex:0];
+    Dprintf("NSDecimalSeparator set (%c)\n", conf.dec_delimiter);
+    if ([[numFormat groupingSeparator] length] > 0) {
+	Dprintf("NSGroupingSeparator > 0\n");
+	conf.thou_delimiter = [[numFormat groupingSeparator] characterAtIndex:0];
     } else {
 	conf.thou_delimiter = 0;
     }
-    Dprintf("NSThousandsSeparator set\n");
-    [decimalKey setAttributedTitle:[prefs objectForKey:NSDecimalSeparator]];
-    Dprintf("decimalKey title set\n");
+    Dprintf("NSGroupingSeparator set (%c)\n", conf.thou_delimiter);
+    if ([prefs integerForKey:@"alternateInputDecimalSeparator"]) {
+	if ([[prefs stringForKey:@"alternateInputDecimalSeparator"] length] > 0) {
+	    conf.in_dec_delimiter = [[prefs stringForKey:@"alternateInputDecimalSeparator"] characterAtIndex:0];
+	}
+	if ([[prefs stringForKey:@"alternateInputThousandsSeparator"] length] > 0) {
+	    conf.in_thou_delimiter = [[prefs stringForKey:@"alternateInputThousandsSeparator"] characterAtIndex:0];
+	}
+	if (conf.in_dec_delimiter != 0) {
+	    [decimalKey setTitle:[NSString stringWithCString:&conf.in_dec_delimiter length:1]];
+	}
+    } else {
+	[decimalKey setTitle:[numFormat decimalSeparator]];
+	Dprintf("decimalKey title set\n");
+    }
 
     /* reset the window to the saved setting */
     superview = [keypad superview];
@@ -505,9 +524,14 @@ static pthread_mutex_t displayLock;
     { // Make the Answerfield big enough to display the answer
 	NSRect curFrame, newFrame;
 	curFrame = [AnswerField frame];
+	Dprintf("displayAnswer - current size: %f x %f\n", curFrame.size.height, curFrame.size.width);
 	newFrame = curFrame;
-	//newFrame.size.height = 10000000.0; // arbitrarily big number
+	// we set the height to a huge number because cellSizeForBounds only
+	// works "within" the specified bound, which we want because we want to
+	// limit the width.
+	newFrame.size.height = DBL_MAX; 
 	newFrame.size = [[AnswerField cell] cellSizeForBounds:newFrame];
+	Dprintf("displayAnswer - needed size: %f x %f\n", newFrame.size.height, newFrame.size.width);
 	if (curFrame.size.height != newFrame.size.height) {
 	    size_t newHeight = newFrame.size.height;
 	    int difference = newHeight - curFrame.size.height;
@@ -749,8 +773,9 @@ static pthread_mutex_t displayLock;
     short need_redraw = 0;
     short olde;
     NSUserDefaults *prefs = [NSUserDefaults standardUserDefaults];
+    int tag = [sender tag];
 
-    switch ([sender tag]) {
+    switch (tag) {
 	case 1: // Flag Undefined Variables
 	    olde = conf.picky_variables;
 	    conf.picky_variables = ([pickyVariables state]==NSOnState);
@@ -942,7 +967,31 @@ static pthread_mutex_t displayLock;
 		[PrecisionSlider setMaxValue:[sliderStepper doubleValue]];
 		[prefs setObject:[NSString stringWithFormat:@"%lu",[sliderStepper intValue]] forKey:@"maxSliderPrecision"];
 	    }
-	    break;    
+	    break;
+	case 24:
+	    [altInputDecSep setEnabled:([sender state] == NSOnState)];
+	    [altInputThouSep setEnabled:([sender state] == NSOnState)];
+	    if ([sender state] != NSOnState) {
+		[altInputDecSep setStringValue:@""];
+		[altInputThouSep setStringValue:@""];
+		[decimalKey setTitle:[NSString stringWithCString:&conf.dec_delimiter length:1]];
+	    }
+	    break;
+	case 25:
+	    if ([[sender stringValue] length] > 0) {
+		conf.in_dec_delimiter = [[sender stringValue] characterAtIndex:0];
+		[decimalKey setTitle:[NSString stringWithCString:&conf.in_dec_delimiter length:1]];
+	    } else {
+		conf.in_dec_delimiter = 0;
+		[decimalKey setTitle:[NSString stringWithCString:&conf.dec_delimiter length:1]];
+	    }
+	    break;
+	case 26:
+	    if ([[sender stringValue] length] > 0) {
+		conf.in_thou_delimiter = [[sender stringValue] characterAtIndex:0];
+	    } else {
+		conf.in_thou_delimiter = 0;
+	    }
 	default: return;
     }
 
@@ -1109,6 +1158,7 @@ static pthread_mutex_t displayLock;
     [cModPref setState:(conf.c_style_mod?NSOnState:NSOffState)];
     [sliderPref setIntValue:(int)[PrecisionSlider maxValue]];
     [sliderStepper setIntValue:(int)[PrecisionSlider maxValue]];
+    [alternateInputPref setState:(conf.in_dec_delimiter != 0 || conf.in_thou_delimiter != 0)];
 
     /* disable irrelevant preferences */
     [historyDuplicates setEnabled:!conf.simple_calc];
@@ -1121,11 +1171,13 @@ static pthread_mutex_t displayLock;
     [engineeringNotation setEnabled:(conf.output_format==DECIMAL_FORMAT)];
     [limitHistoryLen setEnabled:conf.history_limit&&!conf.simple_calc];
     [limitHistoryLenTag setEnabled:conf.history_limit&&!conf.simple_calc];
+    [altInputDecSep setEnabled:[alternateInputPref state]];
+    [altInputThouSep setEnabled:[alternateInputPref state]];
 
-    {
-	char len[6];
-	sprintf(len,"%lu",conf.history_limit_len);
-	[limitHistoryLen setStringValue:[NSString stringWithFormat:@"%s",len]];
+    [limitHistoryLen setStringValue:[NSString stringWithFormat:@"%lu",conf.history_limit_len]];
+    if ([alternateInputPref state]) {
+	[altInputDecSep setStringValue:[NSString stringWithFormat:@"%c",conf.in_dec_delimiter]];
+	[altInputThouSep setStringValue:[NSString stringWithFormat:@"%c",conf.in_thou_delimiter]];
     }
 }
 
