@@ -10,6 +10,26 @@
 #include <float.h>		       /* for DBL_EPSILON */
 #include <ctype.h>		       /* for isalpha() */
 
+#ifndef isnan
+# define isnan(x) \
+    (sizeof (x) == sizeof (long double) ? isnan_ld (x) \
+     : sizeof (x) == sizeof (double) ? isnan_d (x) \
+     : isnan_f (x))
+static inline int isnan_f  (float       x) { return x != x; }
+static inline int isnan_d  (double      x) { return x != x; }
+static inline int isnan_ld (long double x) { return x != x; }
+#endif
+
+#ifndef isinf
+# define isinf(x) \
+    (sizeof (x) == sizeof (long double) ? isinf_ld (x) \
+     : sizeof (x) == sizeof (double) ? isinf_d (x) \
+     : isinf_f (x))
+static inline int isinf_f  (float       x) { return isnan (x - x); }
+static inline int isinf_d  (double      x) { return isnan (x - x); }
+static inline int isinf_ld (long double x) { return isnan (x - x); }
+#endif
+
 #if ! defined(HAVE_CONFIG_H) || HAVE_STRING_H
 # include <string.h>		       /* for memset() */
 #else
@@ -109,19 +129,26 @@ void parseme(const char *pthis)
 
 	for (i = 0; i < strlen(sanitized); ++i) {
 	    if ((conf.thou_delimiter != '.' && conf.dec_delimiter != '.' &&
+		 conf.in_thou_delimiter != '.' && conf.in_dec_delimiter != '.' &&
 		 sanitized[i] == '.') || (conf.thou_delimiter != ',' &&
+					  conf.in_thou_delimiter != ',' &&
 					  conf.dec_delimiter != ',' &&
+					  conf.in_dec_delimiter != ',' &&
 					  sanitized[i] == ',')) {
 		// throw an error
 		report_error("Improperly formatted numbers! (%c,%c)\n",
 			     conf.thou_delimiter, conf.dec_delimiter);
 		synerrors = 1;
 		break;
-	    } else if (sanitized[i] == conf.thou_delimiter)
+	    } else if (conf.in_thou_delimiter != 0 && sanitized[i] == conf.in_thou_delimiter) {
 		sanitized[i] = ',';
-	    else if (sanitized[i] == conf.dec_delimiter)
+	    } else if (conf.in_thou_delimiter == 0 && sanitized[i] == conf.thou_delimiter) {
+		sanitized[i] = ',';
+	    } else if (conf.in_dec_delimiter != 0 && sanitized[i] == conf.in_dec_delimiter) {
 		sanitized[i] = '.';
-	    //      sanitized[i] = conf.charkey[(int)sanitized[i]];
+	    } else if (conf.in_dec_delimiter == 0 && sanitized[i] == conf.dec_delimiter) {
+		sanitized[i] = '.';
+	    }
 	}
     }
 
@@ -720,6 +747,7 @@ static char *print_this_result_dbl(const double result)
 		break;
 	    case HEXADECIMAL_FORMAT:
 		curs = pa + (conf.print_prefixes ? 2 : 0);
+		strswap('.',conf.dec_delimiter,pa);
 	    case OCTAL_FORMAT:
 		curs = pa + (conf.print_prefixes ? 1 : 0);
 		{
@@ -742,6 +770,7 @@ static char *print_this_result_dbl(const double result)
 			not_all_displayed = 0;
 		    }
 		}
+		strswap('.',conf.dec_delimiter,pa);
 		break;
 	    case BINARY_FORMAT:
 	    {
@@ -793,6 +822,7 @@ static char *print_this_result_dbl(const double result)
 		} else {
 		    not_all_displayed = 0;
 		}
+		strswap('.',conf.dec_delimiter,pa);
 	    }			       // binary format
 	}			       /*}}} */
     }				       // if
@@ -914,6 +944,7 @@ char *print_this_result(const Number result)
     if (conf.rounding_indication == NO_ROUNDING_INDICATION) {
 	not_all_displayed = 0;
     }
+    strswap('.',conf.dec_delimiter,pa);
     // add commas
     if (conf.print_commas) {
 	char *str = add_commas(pa, conf.output_format);
@@ -1009,6 +1040,9 @@ void simple_exp(Number output, const Number first, const enum operations op,
 		num_set_ui(temp, 2);
 		num_pow(temp, temp, second);
 		num_div(output, first, temp);
+		if (is_int(first)) {
+		    num_trunc(output, output);
+		}
 		break;
 	    case wmod:
 		if (num_is_zero(second)) {
