@@ -9,13 +9,15 @@
 #ifndef WCALC_CONVERSIONS
 #define WCALC_CONVERSIONS
 
-#include <gmp.h>
-#include <mpfr.h>
+#include <sys/types.h> /* for ssize_t */
+#include <unistd.h> /* also for ssize_t */
+#include "number.h"
 
-void uber_conversion(mpfr_t output, int utype, int fromunit, int tounit,
-		     mpfr_t value);
-int identify_units(char *unit1, char *unit2);
-int unit_id(int utype, char *unit);
+void uber_conversion(Number output, const int utype, const int fromunit, const int tounit,
+		     Number value);
+ssize_t identify_unit(const char *unit);
+ssize_t identify_units(const char *unit1, const char *unit2);
+ssize_t unit_id(const int utype, const char *unit);
 
 struct conversion
 {
@@ -31,7 +33,7 @@ struct conv_req
 };
 
 /* Conversion Types */
-#define MAX_TYPE        8
+#define MAX_TYPE        10
 #define LENGTH_C        0 /*-*/
 #define AREA_C          1 /*-*/
 #define VOLUME_C        2 /*-*/
@@ -40,7 +42,9 @@ struct conv_req
 #define POWER_C         5 /*-*/
 #define FORCE_C         6 /*-*/
 #define ACCELERATION_C  7 /*-*/
-#define TEMPERATURE_C   8 /**/
+#define TEMPERATURE_C   8 /*-*/
+#define ANGLE_C		9 /*-*/
+#define PRESSURE_C	10 /**/
 /* Astronomical Units */
 #define MAX_ASTRONOMICAL_UNIT 5
 #define A_METERS          0
@@ -49,21 +53,25 @@ struct conv_req
 #define LIGHT_YEAR        4
 #define PARSEC            5
 /* Length Units */
-#define LENGTH_UNIT_COUNT		30
+#define LENGTH_UNIT_COUNT		87
 /* Area Units */
-#define AREA_UNIT_COUNT			37
+#define AREA_UNIT_COUNT			74
 /* Volume Units */
-#define VOLUME_UNIT_COUNT		41
+#define VOLUME_UNIT_COUNT		136
+/* Angle Units */
+#define ANGLE_UNIT_COUNT		12
 /* Mass Units */
-#define MASS_UNIT_COUNT			21
+#define MASS_UNIT_COUNT			66
 /* Speed Units */
-#define SPEED_UNIT_COUNT		10
+#define SPEED_UNIT_COUNT		17
 /* Power Units */
-#define POWER_UNIT_COUNT		11
+#define POWER_UNIT_COUNT		45
 /* Force Units */
-#define FORCE_UNIT_COUNT		7
+#define FORCE_UNIT_COUNT		13
 /* Acceleration Units */
-#define ACCELERATION_UNIT_COUNT	18
+#define ACCELERATION_UNIT_COUNT		23
+/* Pressure Units */
+#define PRESSURE_UNIT_COUNT		41
 /* Temperature Units */
 #define TEMPERATURE_UNIT_COUNT	5
 #define KELVIN		0
@@ -71,6 +79,6 @@ struct conv_req
 #define RANKINE		2
 #define FARENHEIT	3
 #define REAUMUR		4
-extern const struct conversion *conversions[MAX_TYPE + 1];
+extern const struct conversion *conversions[MAX_TYPE + 2];
 
 #endif
