@@ -1,27 +1,29 @@
-#include <gmp.h>
-#include <mpfr.h>
+#include "number.h"
 #include "variables.h"
 #include "calculator.h"
 #include "number_formatting.h"
+#ifdef MEMWATCH
+#include "memwatch.h"
+#endif
 
-char *evalvar(char *varname)
+char *evalvar(const char *varname)
 {
     struct answer a;
     char *varvalue, junk;
-    mpfr_t f;
+    Number f;
 
     a = getvar_full(varname);
     if (!a.err) {
-	mpfr_init(f);
+	num_init(f);
 	if (a.exp) {		       // it is an expression
 	    parseme(a.exp);
-	    mpfr_set(f, last_answer, GMP_RNDN);
+	    num_set(f, last_answer);
 	} else {		       // it is a value
-	    mpfr_set(f, a.val, GMP_RNDN);
-	    mpfr_clear(a.val);
+	    num_set(f, a.val);
+	    num_free(a.val);
 	}
 	varvalue = num_to_str_complex(f, 10, 0, -1, 1, &junk);
-	mpfr_clear(f);
+	num_free(f);
 	return varvalue;
     } else {
 	return NULL;
