@@ -16,15 +16,15 @@
 #include <stdlib.h>		       /* for free() */
 #include <limits.h>		       /* for PATH_MAX */
 
-#include <gmp.h>
-#include <mpfr.h>
-
 #include "historyManager.h"
 #include "string_manip.h"
 #include "calculator.h"
 #include "variables.h"
 
 #include "files.h"
+#ifdef MEMWATCH
+#include "memwatch.h"
+#endif
 
 /* this is for communicating with the scanner */
 char *open_file = NULL;
@@ -38,7 +38,7 @@ int saveState(char *filename)
     free(filename);
     if (fd >= 0) {
 	//success
-	int hindex;
+	size_t hindex;
 	int retval;
 	size_t num_vars = numvars();
 
@@ -132,7 +132,7 @@ int saveState(char *filename)
     return return_error;
 }				       /*}}} */
 
-int loadState(char *filename, int into_history)
+int loadState(const char *filename, const int into_history)
 {				       /*{{{ */
     int fd, return_error = 0;
     int standard_output_save = standard_output;
@@ -184,6 +184,7 @@ int loadState(char *filename, int into_history)
 		     conf.remember_errors) && into_history) {
 		    addToHistory(linebuf, last_answer);
 		}
+		free(safe);
 	    }
 	    linelen = 0;
 	    memset(linebuf, 0, maxlinelen);
@@ -195,6 +196,7 @@ int loadState(char *filename, int into_history)
 	if (close(fd) != 0) {
 	    return_error = errno;
 	}
+	free(linebuf);
     } else {
 	return_error = errno;
     }
@@ -202,7 +204,7 @@ int loadState(char *filename, int into_history)
     return return_error;
 }				       /*}}} */
 
-int storeVar(char *variable)
+int storeVar(const char *variable)
 {				       /*{{{ */
     int fd, retval = 0, return_error = 0;
     char filename[PATH_MAX];
@@ -236,7 +238,7 @@ int storeVar(char *variable)
 	    cptr = strdup(print_this_result(keyval.val));
 	}
 	retval = write(fd, cptr, strlen(cptr));
-	if (retval < strlen(cptr)) {
+	if (retval < (int)strlen(cptr)) {
 	    return_error = errno;
 	    free(cptr);
 	    goto exit_storeVar;
@@ -246,7 +248,7 @@ int storeVar(char *variable)
 	    cptr = malloc(strlen(keyval.desc) + 3);
 	    snprintf(cptr, strlen(keyval.desc) + 3, "'%s'", keyval.desc);
 	    retval = write(fd, cptr, strlen(cptr));
-	    if (retval < strlen(cptr)) {
+	    if (retval < (int)strlen(cptr)) {
 		return_error = errno;
 		free(cptr);
 		goto exit_storeVar;
