@@ -9,15 +9,13 @@
 
 #ifndef WCALC_HISTORY_MANAGER
 
-#include <gmp.h>
-#include <mpfr.h>
+#include "number.h"
 
 void clearHistory(void);
-void addToHistory(char *, mpfr_t);
-char * historynum(int,int);
-int historyLength(void);
+void addToHistory(char *, Number);
+char *historynum(int, int);
+size_t historyLength(void);
 
 extern short allow_duplicates;
 extern short recalculate;
 #endif
-
