@@ -1,6 +1,13 @@
 #include <string.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+#include "isfunc.h"
+#ifdef MEMWATCH
+#include "memwatch.h"
+#endif
 
-static char *funcs[] = { "sin", "cos", "tan", "cot",
+const char *funcs[] = { "sin", "cos", "tan", "cot",
     "asin", "acos", "atan", "acot",
     "arcsin", "arccos", "arctan", "arccot",
     "sinh", "cosh", "tanh", "coth",
@@ -10,17 +17,20 @@ static char *funcs[] = { "sin", "cos", "tan", "cot",
     "round", "abs", "floor", "ceil", "ceiling",
     "sqrt", "cbrt",
     "exp", "fact", "comp",
+#ifdef HAVE_MPFR_22
+    "eint","Gamma","gamma","lngamma","lnGamma",
+#endif
     "rand", "irand",
-    "Gamma","lngamma","lnGamma","zeta","sinc",
+    "zeta","sinc",
     0
 };
 
-int isfunc(char *str)
+int isfunc(const char *str)
 {
     size_t i;
 
     for (i = 0; funcs[i]; i++) {
-	if (!strcmp(funcs[i], str)) {
+	if (strcmp(funcs[i], str) == 0) {
 	    return 1;
 	}
     }
