@@ -2,7 +2,9 @@
 * Modified by Kyle Wheeler
 */
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -46,12 +48,17 @@ extern int history_truncate_file(char *, int);
 
 #include "calculator.h"
 #include "conversion.h"
-#include "parser.h"
+#ifdef HAVE_CONFIG_H /* auto-tool build */
+# include "parser.h"
+#else
+# include "y.tab.h"
+#endif
 #include "variables.h"
 #include "help.h"
 #include "files.h"
 #include "historyManager.h"
 #include "list.h"
+#include "isconst.h"
 #ifdef MEMWATCH
 #include "memwatch.h"
 #endif
@@ -103,6 +110,23 @@ char *tc_rounding(const char *text, int state)
     return NULL;
 }				       /*}}} */
 
+/*@null@*/
+char *tc_engineering(const char *text, int state)
+{				       /*{{{ */
+    static unsigned int i = 0;
+    char *engineering[] = { "always", "never", "auto", "automatic", 0 };
+    if (state == 0) {
+	i = 0;
+    }
+    while (engineering[i] != NULL) {
+	if (strncmp(text, engineering[i], strlen(text)) == 0) {
+	    return strdup(engineering[i++]);
+	}
+	i++;
+    }
+    return NULL;
+}				       /*}}} */
+
 #define COMPLETE(strs) do { \
     int compareword = 0; \
     int comparechar = 0; \
@@ -123,12 +147,31 @@ char *tc_rounding(const char *text, int state)
 	} \
     } \
 } while (0)
+#define COMPLETE2(strs) do { \
+    int compareword = 0; \
+    int comparechar = 0; \
+    int textcurs = 0; \
+    while (strs[compareword].label != NULL) { \
+	if (text[textcurs] == 0) { /* add to the list of possibilities */ \
+	    addToList(&tc_options, strdup(strs[compareword].label)); \
+	    textcurs = 0; \
+	    comparechar = 0; \
+	    compareword++; \
+	} else if (text[textcurs] == strs[compareword].label[comparechar]) { \
+	    textcurs++; \
+	    comparechar++; \
+	} else { /* not a possibility: next! */ \
+	    textcurs = 0; \
+	    compareword++; \
+	    comparechar = 0; \
+	} \
+    } \
+} while (0)
 
 char **wcalc_completion(const char *text, int start, int end)
 {				       /*{{{ */
     /*extern const char *commands[];*/
     extern const char *qcommands[];
-    extern const char *consts[];
     extern const char *funcs[];
     char **variables;
     char **retvals = NULL;
@@ -145,7 +188,7 @@ char **wcalc_completion(const char *text, int start, int end)
 		++i;
 	    if (i == start) {
 		COMPLETE(qcommands);
-		COMPLETE(consts);
+		COMPLETE2(consts);
 		COMPLETE(funcs);
 		variables = listvarnames();
 		COMPLETE(variables);
@@ -180,6 +223,23 @@ char **wcalc_completion(const char *text, int start, int end)
 		retvals = rl_completion_matches(text, tc_rounding);
 		return retvals;
 	    }
+	} else
+	    if ((strncmp("\\e", rl_line_buffer, 2) == 0 &&
+	         isspace(rl_line_buffer[2])) ||
+		(strncmp("\\eng", rl_line_buffer, 4) == 0 &&
+		 isspace(rl_line_buffer[4])) ||
+		(strncmp("\\engineering", rl_line_buffer, 12) == 0 &&
+		 isspace(rl_line_buffer[12]))) {
+	    int i = 2;
+
+	    while (!isspace(rl_line_buffer[i]))
+		++i;
+	    while (isspace(rl_line_buffer[i]))
+		++i;
+	    if (i == start) {
+		retvals = rl_completion_matches(text, tc_engineering);
+		return retvals;
+	    }
 	} else
 	    if ((strncmp("\\c", rl_line_buffer, 2) == 0 &&
 		 isspace(rl_line_buffer[2])) ||
@@ -249,7 +309,7 @@ char **wcalc_completion(const char *text, int start, int end)
 	}
     } else {
 	COMPLETE(qcommands);
-	COMPLETE(consts);
+	COMPLETE2(consts);
 	COMPLETE(funcs);
 	variables = listvarnames();
 	COMPLETE(variables);
@@ -288,6 +348,8 @@ int main(int argc, char *argv[])
     conf.picky_variables = 1;
     conf.print_prefixes = 1;
     conf.precision_guard = 1;
+    conf.in_thou_delimiter = 0;
+    conf.in_dec_delimiter = 0;
     conf.thou_delimiter = ',';
     conf.dec_delimiter = '.';
     conf.print_equal = 1;
@@ -350,7 +412,7 @@ int main(int argc, char *argv[])
 	    num_free(last_answer);
 	    exit(EXIT_SUCCESS);
 	} else if (!strcmp(argv[i], "-v") || !strcmp(argv[i], "--version")) {
-	    printf("wcalc %s\n", VERSION);
+	    printf("wcalc "VERSION"\n");
 	    num_free(last_answer);
 	    exit(EXIT_SUCCESS);
 	} else if (!strcmp(argv[i], "-d") || !strcmp(argv[i], "--decimal") ||
@@ -391,6 +453,19 @@ int main(int argc, char *argv[])
 	    fprintf(stderr,
 		    "--round requires an argument (none|simple|sig_fig)\n");
 	    exit(EXIT_FAILURE);
+	} else if (!strncmp(argv[i], "--idsep=", 7)) {
+	    if (strlen(argv[i]) > 8 || strlen(argv[i]) == 7) {
+		fprintf(stderr, "--idsep= must have an argument\n");
+		exit(EXIT_FAILURE);
+	    }
+	    if (conf.in_thou_delimiter != argv[i][7] && (conf.in_thou_delimiter != 0 || conf.thou_delimiter != argv[i][7])) {
+		conf.in_dec_delimiter = argv[i][7];
+	    } else {
+		fprintf(stderr,
+			"%c cannot be the decimal separator. It is the thousands separator.\n",
+			argv[i][7]);
+		exit(EXIT_FAILURE);
+	    }
 	} else if (!strncmp(argv[i], "--dsep=", 7)) {
 	    if (strlen(argv[i]) > 8 || strlen(argv[i]) == 7) {
 		fprintf(stderr, "--dsep= must have an argument\n");
@@ -404,6 +479,19 @@ int main(int argc, char *argv[])
 			argv[i][7]);
 		exit(EXIT_FAILURE);
 	    }
+	} else if (!strncmp(argv[i], "--itsep=", 7)) {
+	    if (strlen(argv[i]) > 8 || strlen(argv[i]) == 7) {
+		fprintf(stderr, "--itsep= must have an argument\n");
+		exit(EXIT_FAILURE);
+	    }
+	    if (conf.in_dec_delimiter != argv[i][7] && (conf.in_dec_delimiter != 0 || conf.dec_delimiter != argv[i][7])) {
+		conf.in_thou_delimiter = argv[i][7];
+	    } else {
+		fprintf(stderr,
+			"%c cannot be the thousands separator. It is the decimal separator.\n",
+			argv[i][7]);
+		exit(EXIT_FAILURE);
+	    }
 	} else if (!strncmp(argv[i], "--tsep=", 7)) {
 	    if (strlen(argv[i]) > 8 || strlen(argv[i]) == 7) {
 		fprintf(stderr, "--tsep= must have an argument\n");
@@ -778,26 +866,22 @@ static int read_prefs(char *filename)
 	while (retval == 1 && curs < BIG_STRING) {
 	    retval = read(fd, &(key[curs]), 1);
 	    if ((!quoted && isspace((int)(key[curs]))) ||
+		(!quoted && key[curs] == '=') ||
 		(quoted && key[curs] != '\''))
 		break;
 	    ++curs;
 	}
 	if (retval != 1)
 	    break;
+	junk = key[curs];
 	key[curs] = 0;
-	// read in the =
-	junk = 0;
-	while (retval == 1 && junk != '=') {
-	    retval = read(fd, &junk, 1);
-	}
-	if (retval != 1)
-	    break;
+	// eat the equals sign (and any surrounding space)
 	while (retval == 1 && (isspace((int)junk) || junk == '=')) {
 	    retval = read(fd, &junk, 1);
 	}
 	if (retval != 1)
 	    break;
-	value[0] = junk;
+	value[0] = junk; // junk now contains the next non-junk character
 	curs = 0;
 	// read in the value
 	quoted = 1;
@@ -834,6 +918,10 @@ static int read_prefs(char *filename)
 	    conf.print_ints = TRUEFALSE;
 	else if (!strcmp(key, "print_delimiters"))
 	    conf.print_commas = TRUEFALSE;
+	else if (!strcmp(key, "input_thousands_delimiter"))
+	    conf.in_thou_delimiter = value[0];
+	else if (!strcmp(key, "input_decimal_delimiter"))
+	    conf.in_dec_delimiter = value[0];
 	else if (!strcmp(key, "thousands_delimiter"))
 	    conf.thou_delimiter = value[0];
 	else if (!strcmp(key, "decimal_delimiter"))
@@ -861,7 +949,7 @@ static int read_prefs(char *filename)
 		conf.rounding_indication = SIMPLE_ROUNDING_INDICATION;
 	    else if (!strcmp(value, "sig_fig"))
 		conf.rounding_indication = SIG_FIG_ROUNDING_INDICATION;
-	else if (!strcmp(key, "engineering"))
+	} else if (!strcmp(key, "engineering")) {
 	    if (!strcmp(value,"auto") || !strcmp(value,"automatic") || !strcmp(value,"yes") || !strcmp(value,"true") || !strcmp(value,"1")) {
 		conf.engineering = automatic;
 	    } else if (!strcmp(value, "always")) {
