@@ -6,12 +6,13 @@
 # include <string.h>
 #endif
 
-#include <gmp.h>
-#include <mpfr.h>
-
+#include "number.h"
 #include "calculator.h"		       /* for report_error */
 #include "list.h"
 #include "variables.h"
+#ifdef MEMWATCH
+#include "memwatch.h"
+#endif
 
 #define THE_VALUE 0
 #define THE_STRUCTURE 2
@@ -21,7 +22,7 @@
 List them = NULL;
 
 /* Hidden, internal functions */
-static void *getvar_core(char *key, int all_or_nothing);
+static void *getvar_core(const char *key, const int all_or_nothing);
 
 void initvar(void)
 {				       /*{{{ */
@@ -37,7 +38,7 @@ void cleanupvar(void)
 	if (freeme->exp == 1) {
 	    free(freeme->expression);
 	} else {
-	    mpfr_clear(freeme->value);
+	    num_free(freeme->value);
 	}
 	if (freeme->description) {
 	    free(freeme->description);
@@ -52,7 +53,49 @@ size_t numvars()
     return listLen(them);
 }				       /*}}} */
 
-void delnvar(size_t i)
+/* prints out all the variables */
+void printvariables(void)
+{/*{{{*/
+    ListIterator li = NULL;
+    struct variable *cursor = NULL;
+
+    if (!them || listLen(them) == 0)
+	return;
+
+    li = getListIterator(them);
+    while ((cursor = (struct variable *)nextListElement(li)) != NULL) {
+	printf("%s = ", cursor->key);
+	if (cursor->exp) {
+	    printf("%s\n", cursor->expression);
+	} else {
+	    printf("%g\n", num_get_d(cursor->value));
+	}
+    }
+    freeListIterator(li);
+}/*}}}*/
+
+/* returns a list of variables (only the base array is malloc'd, the rest must
+ * NOT be freed */
+char **listvarnames(void)
+{/*{{{*/
+    size_t i;
+    char **ret = calloc(listLen(them) + 1, sizeof(char *));
+    ListIterator li = NULL;
+    struct variable *cursor = NULL;
+
+    if (!them || listLen(them) == 0)
+	return ret;
+
+    li = getListIterator(them);
+    i = 0;
+    while ((cursor = (struct variable *)nextListElement(li)) != NULL) {
+	ret[i++] = cursor->key;
+    }
+    freeListIterator(li);
+    return ret;
+/*}}}*/}
+
+void delnvar(const size_t i)
 {				       /*{{{ */
     struct variable *freeme;
 
@@ -62,24 +105,24 @@ void delnvar(size_t i)
 	if (freeme->exp == 1) {
 	    free(freeme->expression);
 	} else {
-	    mpfr_clear(freeme->value);
+	    num_free(freeme->value);
 	}
 	free(freeme);
     }
 }				       /*}}} */
 
-struct variable *getrealnvar(size_t i)
+struct variable *getrealnvar(const size_t i)
 {				       /*{{{ */
     return (struct variable *)peekListElement(them, i);
 }				       /*}}} */
 
-struct answer getvar(char *key)
+struct answer getvar(const char *key)
 {				       /*{{{ */
     struct answer ans;
-    mpfr_t *t = getvar_core(key, THE_VALUE);
+    Number *t = getvar_core(key, THE_VALUE);
 
     if (t) {
-	mpfr_init_set(ans.val, *t, GMP_RNDN);
+	num_init_set(ans.val, *t);
 	ans.err = 0;
 	ans.exp = NULL;
     } else {
@@ -91,23 +134,23 @@ struct answer getvar(char *key)
     return ans;
 }				       /*}}} */
 
-void getvarval(mpfr_t out, char *key)
+void getvarval(Number out, const char *key)
 {				       /*{{{ */
-    mpfr_t *t = getvar_core(key, THE_VALUE);
+    Number *t = getvar_core(key, THE_VALUE);
 
     if (t) {
-	mpfr_set(out, *t, GMP_RNDN);
+	num_set(out, *t);
     }
 }				       /*}}} */
 
-struct answer getvar_full(char *key)
+struct answer getvar_full(const char *key)
 {				       /*{{{ */
     struct answer ans;
     struct variable *var;
 
     var = getvar_core(key, THE_STRUCTURE);
     if (var && !var->exp) {
-	mpfr_init_set(ans.val, var->value, GMP_RNDN);
+	num_init_set(ans.val, var->value);
 	ans.err = 0;
 	ans.exp = NULL;
 	ans.desc = var->description;
@@ -122,14 +165,9 @@ struct answer getvar_full(char *key)
     return ans;
 }				       /*}}} */
 
-struct variable *getvarptr(char *key)
-{				       /*{{{ */
-    return (struct variable *)getvar_core(key, THE_STRUCTURE);
-}				       /*}}} */
-
 /* This function returns 1 if a variable by that key has already been created
  * and returns 0 if a variable by that key has not been created yet */
-int varexists(char *key)
+int varexists(const char *key)
 {				       /*{{{ */
     struct variable *cursor = NULL;
     ListIterator li = NULL;
@@ -148,7 +186,7 @@ int varexists(char *key)
     return cursor ? 1 : 0;
 }				       /*}}} */
 
-static void *getvar_core(char *key, int all_or_nothing)
+static void *getvar_core(const char *key, const int all_or_nothing)
 {				       /*{{{ */
     struct variable *cursor = NULL;
     ListIterator li = NULL;
@@ -184,7 +222,7 @@ static void *getvar_core(char *key, int all_or_nothing)
 
 /* this adds the key-expression pair to the list.
  * if the key already exists, change the value to this */
-int putexp(char *key, char *value, char *desc)
+int putexp(const char *key, const char *value, const char *desc)
 {				       /*{{{ */
     struct variable *cursor = NULL;
 
@@ -202,7 +240,7 @@ int putexp(char *key, char *value, char *desc)
 	if (cursor->expression) {
 	    free(cursor->expression);
 	} else {
-	    mpfr_clear(cursor->value);
+	    num_free(cursor->value);
 	}
 	if (cursor->description) {
 	    free(cursor->description);
@@ -223,7 +261,7 @@ int putexp(char *key, char *value, char *desc)
 
 /* this adds the key-value pair to the list.
  * if the key already exists, change the value to this */
-int putval(char *key, mpfr_t value, char *desc)
+int putval(const char *key, const Number value, const char *desc)
 {				       /*{{{ */
     struct variable *cursor = NULL;
 
@@ -241,11 +279,11 @@ int putval(char *key, mpfr_t value, char *desc)
     // but key may not exist, and value may not be properly initialized
     if (!cursor->key) {
 	cursor->key = (char *)strdup(key);
-	mpfr_init(cursor->value);
+	num_init(cursor->value);
     } else if (cursor->expression) {
 	free(cursor->expression);
 	cursor->expression = NULL;
-	mpfr_init(cursor->value);
+	num_init(cursor->value);
     }
     if (cursor->description) {
 	free(cursor->description);
@@ -258,6 +296,6 @@ int putval(char *key, mpfr_t value, char *desc)
     } else {
 	cursor->description = NULL;
     }
-    mpfr_set(cursor->value, value, GMP_RNDN);
+    num_set(cursor->value, value);
     return 0;
 }				       /*}}} */
